/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : May. 2007

 File          : WakeUpTimer.c

 Hardware      : Applicable to ADuC702x rev H or I silicon
                 Currently targetting ADuC7028.

 Description   : Wake-up Timer causes part to power-up, perform an Analog 
 				 to Digital Conversion, output conversion via UART at 9600bps 
				 and power-down again, once every minute. 

				 the debugger cannot be used when the part is placed in power down mode.
		
*********************************************************************/

#include <ADuC7028.h>

void My_IRQ_Function(void);
void senddata(short);
void ADCpoweron(int);
char hex2ascii(char);
int t;							// Used for UART send data delay in IRQ function

int main(void)
{
	POWKEY1 = 0x01;
	POWCON = 0x00;		   			// 41.78MHz
	POWKEY2 = 0xF4;

	GP4DAT = 0x04000000;			// Configure P4.2 as output
	GP4DAT ^= 0x00040000;			// Complement P4.2 	
		
	GP1CON = 0x011;					// Setup tx & rx pins on P1.0 and P1.1
	
	// Start setting up UART at 9600bps (CD = 0 in startup.s)
	COMCON0 = 0x80;					// Setting DLAB
	COMDIV0 = 0x88;					// 
   	COMDIV1 = 0x00;
   	COMCON0 = 0x07;					// Clearing DLAB

	T2CON = 0xE8;					// Configures Timer2 to count down, with 
									// "Hr:Min:Sec:Hundredths" format and prescalar of 256

   	T2LD = 0x0005;				
//	T2LD = 0x00010000;				// Sets the counter register to 5 minutes. Format: HH:MM:SS:hh

	IRQ = My_IRQ_Function;			// 	Specify Interrupt Service Rountine 

	IRQEN = WAKEUP_TIMER_BIT;		//	Enable wake up timer IRQ 

/*	POWKEY1 = 0x01;
	POWCON = 0x30;		   			// Device in sleep mode
	POWKEY2 = 0xF4;
 */	GP4DAT ^= 0x00040000;			// Complement P4.2 	

	while (1){
	}

}


/********************************************************************/
/*                                                                  */
/*		Interrupt Service Rountine									*/
/*                                                                  */
/********************************************************************/

void My_IRQ_Function()
{
	GP4DAT ^= 0x00040000;			// Complement P4.2 each time an interrupt 
									// occurs (light on during conversion)	
	
	//ADC configuration
	ADCpoweron(20000);				// power on ADC										
	REFCON = 0x01;					// connect internal 2.5V reference to VREF pin
	ADCCP  = 0x00;				    // Selecting ADC Channel 0	
	ADCCON = 0x4E3;					// Perform single software conversion (ADC Speed = 1 MSPS)
	
	while (!ADCSTA){}				// wait for end of conversion
		senddata(ADCDAT >> 16); 	// Output Conversion

	ADCCON = 0x00;					// Powers Down ADC
	
	t = 5000;					 	// Delay required to send data via UART
	while(t>0)
	{
		t--;
	}

	GP4DAT ^= 0x00040000;			// Complement P4.2 at end of conversion (switches off)

	T2CLRI = 0xFF;	  				// Clearing Timer2 Interrupt

/*	POWKEY1 = 0x01;
	POWCON = 0x30;		   			// Device in sleep mode
	POWKEY2 = 0xF4;
*/
	return;
}





void ADCpoweron(int time)
{
	ADCCON = 0x20;	 					// power-on the ADC
	while (time >=0)	  				// wait for ADC to be fully powered on
    time--;
}


void senddata(short to_send)
{
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = 0x0A;						// output LF 
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = 0x0D;						// output CR 
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = hex2ascii ((to_send >> 8) & 0x0F);
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = hex2ascii ((to_send >> 4) & 0x0F);						
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = hex2ascii (to_send & 0x0F);	
}



char hex2ascii(char toconv)
{
	if (toconv<0x0A) 
	{
		toconv += 0x30;
	}
	else 
	{
		toconv += 0x37;
	}	
	return (toconv);
}
